# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Resource definitions for Cloud Platform Apis generated from apitools."""

import enum


BASE_URL = 'https://dataplex.googleapis.com/v1/'
DOCS_URL = 'https://cloud.google.com/dataplex/docs'


class Collections(enum.Enum):
  """Collections for all supported apis."""

  ORGANIZATIONS = (
      'organizations',
      'organizations/{organizationsId}',
      {},
      ['organizationsId'],
      True
  )
  ORGANIZATIONS_LOCATIONS = (
      'organizations.locations',
      'organizations/{organizationsId}/locations/{locationsId}',
      {},
      ['organizationsId', 'locationsId'],
      True
  )
  ORGANIZATIONS_LOCATIONS_ENCRYPTIONCONFIGS = (
      'organizations.locations.encryptionConfigs',
      '{+name}',
      {
          '':
              'organizations/{organizationsId}/locations/{locationsId}/'
              'encryptionConfigs/{encryptionConfigsId}',
      },
      ['name'],
      True
  )
  ORGANIZATIONS_LOCATIONS_OPERATIONS = (
      'organizations.locations.operations',
      '{+name}',
      {
          '':
              'organizations/{organizationsId}/locations/{locationsId}/'
              'operations/{operationsId}',
      },
      ['name'],
      True
  )
  PROJECTS = (
      'projects',
      'projects/{projectsId}',
      {},
      ['projectsId'],
      True
  )
  PROJECTS_LOCATIONS = (
      'projects.locations',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_ASPECTTYPES = (
      'projects.locations.aspectTypes',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/aspectTypes/'
              '{aspectTypesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_CHANGEREQUESTS = (
      'projects.locations.changeRequests',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/changeRequests/'
              '{changeRequestsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_DATAATTRIBUTEBINDINGS = (
      'projects.locations.dataAttributeBindings',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/'
              'dataAttributeBindings/{dataAttributeBindingsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_DATADOMAINS = (
      'projects.locations.dataDomains',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/dataDomains/'
              '{dataDomainsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_DATADOMAINS_BINDINGS = (
      'projects.locations.dataDomains.bindings',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/dataDomains/'
              '{dataDomainsId}/bindings/{bindingsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_DATAPRODUCTS = (
      'projects.locations.dataProducts',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/dataProducts/'
              '{dataProductsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_DATAPRODUCTS_DATAASSETS = (
      'projects.locations.dataProducts.dataAssets',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/dataProducts/'
              '{dataProductsId}/dataAssets/{dataAssetsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_DATASCANS = (
      'projects.locations.dataScans',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/dataScans/'
              '{dataScansId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_DATASCANS_JOBS = (
      'projects.locations.dataScans.jobs',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/dataScans/'
              '{dataScansId}/jobs/{jobsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_DATATAXONOMIES = (
      'projects.locations.dataTaxonomies',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/dataTaxonomies/'
              '{dataTaxonomiesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_DATATAXONOMIES_ATTRIBUTES = (
      'projects.locations.dataTaxonomies.attributes',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/dataTaxonomies/'
              '{dataTaxonomiesId}/attributes/{attributesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_ENTRYGROUPS = (
      'projects.locations.entryGroups',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/entryGroups/'
              '{entryGroupsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_ENTRYGROUPS_ENTRIES = (
      'projects.locations.entryGroups.entries',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/entryGroups/'
              '{entryGroupsId}/entries/{entriesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_ENTRYGROUPS_ENTRYLINKS = (
      'projects.locations.entryGroups.entryLinks',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/entryGroups/'
              '{entryGroupsId}/entryLinks/{entryLinksId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_ENTRYTYPES = (
      'projects.locations.entryTypes',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/entryTypes/'
              '{entryTypesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_GLOSSARIES = (
      'projects.locations.glossaries',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/glossaries/'
              '{glossariesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_GLOSSARIES_CATEGORIES = (
      'projects.locations.glossaries.categories',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/glossaries/'
              '{glossariesId}/categories/{categoriesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_GLOSSARIES_TERMS = (
      'projects.locations.glossaries.terms',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/glossaries/'
              '{glossariesId}/terms/{termsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_LAKES = (
      'projects.locations.lakes',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/lakes/{lakesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_LAKES_ACTIONS = (
      'projects.locations.lakes.actions',
      'projects/{projectsId}/locations/{locationsId}/lakes/{lakesId}/actions/'
      '{actionsId}',
      {},
      ['projectsId', 'locationsId', 'lakesId', 'actionsId'],
      True
  )
  PROJECTS_LOCATIONS_LAKES_TASKS = (
      'projects.locations.lakes.tasks',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/lakes/{lakesId}/'
              'tasks/{tasksId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_LAKES_TASKS_JOBS = (
      'projects.locations.lakes.tasks.jobs',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/lakes/{lakesId}/'
              'tasks/{tasksId}/jobs/{jobsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_LAKES_ZONES = (
      'projects.locations.lakes.zones',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/lakes/{lakesId}/'
              'zones/{zonesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_LAKES_ZONES_ACTIONS = (
      'projects.locations.lakes.zones.actions',
      'projects/{projectsId}/locations/{locationsId}/lakes/{lakesId}/zones/'
      '{zonesId}/actions/{actionsId}',
      {},
      ['projectsId', 'locationsId', 'lakesId', 'zonesId', 'actionsId'],
      True
  )
  PROJECTS_LOCATIONS_LAKES_ZONES_ASSETS = (
      'projects.locations.lakes.zones.assets',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/lakes/{lakesId}/'
              'zones/{zonesId}/assets/{assetsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_LAKES_ZONES_ASSETS_ACTIONS = (
      'projects.locations.lakes.zones.assets.actions',
      'projects/{projectsId}/locations/{locationsId}/lakes/{lakesId}/zones/'
      '{zonesId}/assets/{assetsId}/actions/{actionsId}',
      {},
      ['projectsId', 'locationsId', 'lakesId', 'zonesId', 'assetsId', 'actionsId'],
      True
  )
  PROJECTS_LOCATIONS_LAKES_ZONES_ENTITIES = (
      'projects.locations.lakes.zones.entities',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/lakes/{lakesId}/'
              'zones/{zonesId}/entities/{entitiesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_LAKES_ZONES_ENTITIES_PARTITIONS = (
      'projects.locations.lakes.zones.entities.partitions',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/lakes/{lakesId}/'
              'zones/{zonesId}/entities/{entitiesId}/partitions/'
              '{partitionsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_METADATAFEEDS = (
      'projects.locations.metadataFeeds',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/metadataFeeds/'
              '{metadataFeedsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_METADATAJOBS = (
      'projects.locations.metadataJobs',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/metadataJobs/'
              '{metadataJobsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_OPERATIONS = (
      'projects.locations.operations',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/operations/'
              '{operationsId}',
      },
      ['name'],
      True
  )

  def __init__(self, collection_name, path, flat_paths, params,
               enable_uri_parsing):
    self.collection_name = collection_name
    self.path = path
    self.flat_paths = flat_paths
    self.params = params
    self.enable_uri_parsing = enable_uri_parsing



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""The meta cache completers run command."""


import sys

from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import parser_extensions
from googlecloudsdk.command_lib.meta import cache_util
from googlecloudsdk.command_lib.util import parameter_info_lib
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.command_lib.util.concepts import presentation_specs
from googlecloudsdk.core import exceptions
from googlecloudsdk.core import log
from googlecloudsdk.core import module_util
from googlecloudsdk.core.console import console_io


class _FunctionCompleter:
  """Convert an argparse function completer to a resource_cache completer."""

  def __init__(self, completer):
    self._completer = completer
    self.parameters = None

  def ParameterInfo(self, parsed_args, argument):
    del argument
    return parsed_args

  def Complete(self, prefix, parameter_info):
    return self._completer(prefix, parsed_args=parameter_info)


def _GetPresentationSpec(resource_spec_path, **kwargs):
  """Build a presentation spec."""
  resource_spec = module_util.ImportModule(resource_spec_path)
  if callable(resource_spec):
    resource_spec = resource_spec()
  flag_name_overrides = kwargs.pop('flag_name_overrides', '')
  flag_name_overrides = {
      o.split(':')[0]: o.split(':')[1] if ':' in o else ''
      for o in flag_name_overrides.split(';')
      if o}
  prefixes = kwargs.pop('prefixes', False)
  return presentation_specs.ResourcePresentationSpec(
      kwargs.pop('name', resource_spec.name),
      resource_spec,
      'help text',
      flag_name_overrides=flag_name_overrides,
      prefixes=prefixes,
      **kwargs)


def _GetCompleter(module_path, cache=None, qualify=None,
                  resource_spec=None, presentation_kwargs=None, attribute=None,
                  **kwargs):
  """Returns an instantiated completer for module_path."""
  presentation_kwargs = presentation_kwargs or {}
  if resource_spec:
    presentation_spec = _GetPresentationSpec(resource_spec,
                                             **presentation_kwargs)
    completer = module_util.ImportModule(module_path)(
        presentation_spec.concept_spec,
        attribute)

  else:
    completer = module_util.ImportModule(module_path)
    if not isinstance(completer, type):
      return _FunctionCompleter(completer)
  try:
    return completer(
        cache=cache,
        qualified_parameter_names=qualify,
        **kwargs)
  except TypeError:
    return _FunctionCompleter(completer())


class AddCompleterResourceFlags(parser_extensions.DynamicPositionalAction):
  """Adds resource argument flags based on the completer."""

  def __init__(self, *args, **kwargs):
    super().__init__(*args, **kwargs)
    self.__argument = None
    self.__completer = None

  def GenerateArgs(self, namespace, module_path):
    args = []
    presentation_kwargs = namespace.resource_presentation_kwargs or {}
    # Add the args that correspond to the resource arg, but make them
    # non-required.
    if namespace.resource_spec_path:
      spec = _GetPresentationSpec(namespace.resource_spec_path,
                                  **presentation_kwargs)
      info = concept_parsers.ConceptParser([spec]).GetInfo(spec.name)
      for arg in info.GetAttributeArgs():
        if arg.name.startswith('--'):
          arg.kwargs['required'] = False
        else:
          arg.kwargs['nargs'] = '?' if not spec.plural else '*'
        args.append(arg)
    kwargs = namespace.kwargs or {}
    self.__completer = _GetCompleter(
        module_path, qualify=namespace.qualify,
        resource_spec=namespace.resource_spec_path,
        presentation_kwargs=presentation_kwargs,
        attribute=namespace.attribute,
        **kwargs)
    if self.__completer.parameters:
      for parameter in self.__completer.parameters:
        dest = parameter_info_lib.GetDestFromParam(parameter.name)
        if hasattr(namespace, dest):
          # Don't add if its already been added.
          continue
        flag = parameter_info_lib.GetFlagFromDest(dest)
        arg = base.Argument(
            flag,
            dest=dest,
            category='RESOURCE COMPLETER',
            help=(
                f'{self.__completer.__class__.__name__} `{parameter.name}` '
                'parameter value.'))
        args.append(arg)
    self.__argument = base.Argument(
        'resource_to_complete',
        nargs='?',
        help=('The partial resource name to complete. Omit to enter an '
              'interactive loop that reads a partial resource name from the '
              'input and lists the possible prefix matches on the output '
              'or displays an ERROR message.'))
    args.append(self.__argument)
    return args

  def Completions(self, prefix, parsed_args, **kwargs):
    parameter_info = self.__completer.ParameterInfo(
        parsed_args, self.__argument)
    return self.__completer.Complete(prefix, parameter_info)


class Run(base.Command):
  """Cloud SDK completer module tester.

  *{command}* is an ideal way to debug completer modules without interference
  from the shell.  Shells typically ignore completer errors by disabling all
  standard output, standard error and exception messaging.  Specify
  `--verbosity=INFO` to enable completion and resource cache tracing.
  """

  @staticmethod
  def Args(parser):
    # Add a concept handler that will be stuffed dynamically with information.
    concept_parsers.ConceptParser([]).AddToParser(parser)
    parser.add_argument(
        '--resource-spec-path',
        help=('The resource spec path for a resource argument auto-generated '
              'completer.'))
    parser.add_argument(
        '--attribute',
        help=('The name of the resource attribute for a resource argument '
              'auto-generated completer.'))
    parser.add_argument(
        '--resource-presentation-kwargs',
        type=arg_parsers.ArgDict(
            spec={
                'name': str,
                'flag_name_overrides': str,
                'plural': bool,
                'prefixes': bool,
                'required': bool}),
        help=('Dict of kwargs to be passed to the presentation spec for the '
              'resource argument for which a completer is being tested, such '
              'as name, prefixes, plural, flag name overrides (format as a '
              'list of semicolon-separated key:value pairs). Prefixes is False '
              'by default. Name is the resource spec name by default.'))
    cache_util.AddCacheFlag(parser)
    parser.add_argument(
        '--qualify',
        metavar='NAME',
        type=arg_parsers.ArgList(),
        help=('A list of resource parameter names that must always be '
              'qualified. This is a manual setting for testing. The CLI sets '
              'this automatically.'))
    parser.add_argument(
        '--kwargs',
        metavar='NAME=VALUE',
        type=arg_parsers.ArgDict(),
        help=('Keyword arg dict passed to the completer constructor. For '
              'example, use this to set the resource collection and '
              'list command for `DeprecatedListCommandCompleter`:\n\n'
              '  --kwargs=collection=...,foo="..."'))
    parser.add_argument(
        '--stack-trace',
        action='store_true',
        default=True,
        help=('Enable all exception stack traces, including Cloud SDK core '
              'exceptions.'))
    parser.AddDynamicPositional(
        'module_path',
        action=AddCompleterResourceFlags,
        help=('The completer module path. Run $ gcloud meta completers list` '
              'to list the module paths of the available completers. A '
              'completer module may declare additional flags. Specify `--help` '
              'after _MODULE_PATH_ for details on the module specific flags.'
              '\n\nNOTE: To test resource argument completers, use the '
              'module path "googlecloudsdk.command_lib.util.completers:'
              'CompleterForAttribute". The flags `--resource-spec-path`, '
              '`--attribute`, and (if desired) `--resource-presentation-'
              'kwargs` must be provided BEFORE the positional. Unlike with '
              'most gcloud commands, the arguments are generated on the fly '
              'using the completer you provide, so all the information to '
              'create a resource completer needs to be provided up-front. For '
              'example:\n\n  $ {command} --resource-spec-path MODULE_PATH:'
              'SPEC_OBJECT --attribute ATTRIBUTE_NAME --resource-presentation-'
              'kwargs flag_name_overrides=ATTRIBUTE1:FLAG1;ATTRIBUTE2:FLAG2 '
              'googlecloudsdk.command_lib.util.completers:CompleterForAttribute'
             ))

  def Run(self, args):
    """Returns the results for one completion."""
    presentation_kwargs = args.resource_presentation_kwargs or {}
    with cache_util.GetCache(args.cache, create=True) as cache:
      log.info(f'cache name {cache.name}')
      if not args.kwargs:
        args.kwargs = {}
      # Create the ResourceInfo object that is used to hook up the parameter
      # info to the argparse namespace for resource argument completers.
      if args.resource_spec_path:
        spec = _GetPresentationSpec(
            args.resource_spec_path,
            **presentation_kwargs)
        spec.required = False
        resource_info = concept_parsers.ConceptParser([spec]).GetInfo(spec.name)
        # Since the argument being completed doesn't have the correct
        # dest, make sure the handler always gives the same ResourceInfo
        # object.
        def ResourceInfoMonkeyPatch(*args, **kwargs):
          del args, kwargs
          return resource_info
        args.CONCEPTS.ArgNameToConceptInfo = ResourceInfoMonkeyPatch

      completer = _GetCompleter(
          args.module_path, cache=cache, qualify=args.qualify,
          resource_spec=args.resource_spec_path,
          presentation_kwargs=presentation_kwargs,
          attribute=args.attribute,
          **args.kwargs)
      parameter_info = completer.ParameterInfo(
          args, args.GetPositionalArgument('resource_to_complete'))
      if args.resource_to_complete is not None:
        matches = completer.Complete(args.resource_to_complete, parameter_info)
        return [matches]
      while True:
        name = console_io.PromptResponse('COMPLETE> ')
        if name is None:
          break
        try:
          completions = completer.Complete(name, parameter_info)
        except (Exception, SystemExit) as e:  # pylint: disable=broad-except
          if args.stack_trace:
            exceptions.reraise(Exception(e))
          else:
            log.error(str(e))
          continue
        if completions:
          print('\n'.join(completions))
      sys.stderr.write('\n')
      return None



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Initialize a Backup and DR Service Config."""


from apitools.base.py import exceptions as apitools_exceptions
from googlecloudsdk.api_lib.backupdr import util
from googlecloudsdk.api_lib.backupdr.service_config import ServiceConfigClient
from googlecloudsdk.api_lib.util import exceptions
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.backupdr import flags
from googlecloudsdk.core import log


@base.DefaultUniverseOnly
@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA, base.ReleaseTrack.GA
)
class Init(base.Command):
  """Initialize a Backup and DR Service configuration."""

  detailed_help = {
      'BRIEF': 'Initializes a Backup and DR Service configuration.',
      'DESCRIPTION': '{description}',
      'API REFERENCE': (
          'This command uses the backupdr/v1 API. The full documentation for'
          ' this API can be found at:'
          ' https://cloud.google.com/backup-disaster-recovery'
      ),
      'EXAMPLES': """\
        To initialize a new service configuration in location ``MY_LOCATION''
        and project ``MY_PROJECT'' for resource type ``MY_RESOURCE_TYPE'', run:

        $ {command} --project=MY_PROJECT \
            --location=MY_LOCATION \
            --resource-type=MY_RESOURCE_TYPE
        """,
  }

  @staticmethod
  def Args(parser):
    """Specifies additional command flags.

    Args:
      parser: argparse.Parser: Parser object for command line inputs.
    """
    flags.AddNoAsyncFlag(parser)
    flags.AddLocationResourceArg(
        parser,
        """The location for which the service configuration should be created.""",
    )
    flags.AddResourceType(
        parser,
        """The resource type to which the default service configuration
           will be applied. Examples include, "compute.<UNIVERSE_DOMAIN>.com/Instance"
""",
    )
    flags.AddCloudsqlEdition(
        parser,
        """Cloud SQL Edition: (Required only for Cloud SQL instances, ignored
        otherwise). The edition of the Cloud SQL instance. Possible values:
        "ENTERPRISE" - Enterprise edition.
        "ENTERPRISE_PLUS" - Enterprise Plus edition.""",
    )

  def Run(self, args):
    """Constructs and sends request.

    Args:
      args: argparse.Namespace, An object that contains the values for the
        arguments specified in the .Args() method.

    Returns:
      ProcessHttpResponse of the request made.
    """
    api_version = util.GetApiVersion(self.ReleaseTrack())
    client = ServiceConfigClient(api_version=api_version)
    location = args.CONCEPTS.location.Parse().RelativeName()
    resource_type = args.resource_type
    cloudsql_edition = args.cloudsql_edition
    no_async = args.no_async

    try:
      operation = client.Init(location, resource_type, cloudsql_edition)
    except apitools_exceptions.HttpError as e:
      raise exceptions.HttpException(e, util.HTTP_ERROR_FORMAT)

    if no_async:
      resource = client.WaitForOperation(
          operation_ref=client.GetOperationRef(operation),
          message=(
              'Initializing service configuration for resource type [{}] in'
              ' location [{}]. (This operation could take up to 2 minutes.)'
              .format(resource_type, location)
          ),
          has_result=False,
      )

      # pylint: disable=protected-access
      # none of the log.CreatedResource, log.DeletedResource etc. matched
      log._PrintResourceChange(
          'Initialization of service configuration',
          location,
          kind='location',
          is_async=False,
          details=None,
          failed=None,
          operation_past_tense='Service configuration initialized for',
      )
      return resource

    # pylint: disable=protected-access
    # none of the log.CreatedResource, log.DeletedResource etc. matched
    log._PrintResourceChange(
        'Initialization of service configuration',
        location,
        kind='location',
        is_async=True,
        details=util.ASYNC_OPERATION_MESSAGE.format(operation.name),
        failed=None,
    )
    return operation



# -*- coding: utf-8 -*- #
# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Supporting libraries for the config-helper command."""


from googlecloudsdk.core import config


class ConfigHelperResult(object):
  """The result of the gcloud config config-helper command that gets serialzied.

  Attributes:
    credential: Credential, The OAuth2 credential information.
    configuration: Configuration, Local Cloud SDK configuration information.
    sentinels: Sentinels, Paths to various sentinel files.
  """

  def __init__(self, credential, active_configuration, properties):
    self.credential = Credential(credential)
    self.configuration = Configuration(active_configuration, properties)
    self.sentinels = Sentinels(config.Paths().config_sentinel_file)


class Credential(object):
  """Holder for credential data.

  Attributes:
    access_token: str, The current OAuth2 access token.
    token_expiry: str, The expiry time in UTC as an RFC3339 formatted string.
    id_token: str, The current OAuth2 identity token, if present.
    regional_access_boundary: str, The current regional access boundary, if
      present. This is a hex-encoded bitmap string (e.g. '0xA30') representing a
      list of regions the credential is allowed to access.
    regional_access_boundary_expiry: str, The expiry time of the regional access
      boundary as an RFC3339 formatted string, if present.
  """
  _EXPIRY_FORMAT = '%Y-%m-%dT%H:%M:%SZ'

  def __init__(self, cred):
    self.access_token = cred.token
    expiry = getattr(cred, 'expiry', None)

    self.token_expiry = (
        expiry.strftime(Credential._EXPIRY_FORMAT) if expiry else None)

    # The cache blanks the token_response field, so if it's present that
    # indicates there's either no cache entry, or we just refreshed tokens.
    # Either way, the response is fresher.
    token_response = getattr(cred, 'token_response', None)

    if token_response:
      id_token = token_response.get('id_token', None)
    else:
      id_token = getattr(cred, 'id_tokenb64', None)

    self.id_token = id_token
    self.regional_access_boundary = getattr(
        cred, 'regional_access_boundary', None
    )
    rab_expiry = getattr(cred, 'regional_access_boundary_expiry', None)
    self.regional_access_boundary_expiry = (
        rab_expiry.strftime(Credential._EXPIRY_FORMAT) if rab_expiry else None
    )


class Configuration(object):
  """Holder for configuration data.

  Attributes:
    active_configuration: str, The name of the active configuration.
    properties: {str: {str: str}}, A dict of section names to properties and
      values.
  """

  def __init__(self, active_configuration, properties):
    self.active_configuration = active_configuration
    self.properties = properties


class Sentinels(object):
  """Holder for sentinel file locations.

  Attributes:
    config_sentinel: str, The path to the sentinel that indicates changes were
      made to properties or the active configuration.
  """

  def __init__(self, config_sentinel):
    self.config_sentinel = config_sentinel



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Creates a new Backup Plan."""


import textwrap

from apitools.base.py import exceptions as apitools_exceptions
from googlecloudsdk.api_lib.backupdr import backup_plans
from googlecloudsdk.api_lib.backupdr import util
from googlecloudsdk.api_lib.util import exceptions
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.backupdr import flags
from googlecloudsdk.core import log


@base.DefaultUniverseOnly
@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA, base.ReleaseTrack.GA
)
class Create(base.CreateCommand):
  """Creates a new Backup Plan."""

  detailed_help = {
      'BRIEF': 'Creates a new backup plan',
      'DESCRIPTION': """\
          Create a new backup plan in the project. It can only be created in
          locations where Backup and DR is available.
      """,
      'EXAMPLES': """\
        To create a new backup plan ``sample-backup-plan'' for Compute Engine instances
        in project ``sample-project'',
        at location ``us-central1'',
        with resource-type ``compute.<UNIVERSE_DOMAIN>/Instance'' and
        backup-vault ``backup-vault''
        with 2 backup-rules:

        run:

          $ {command} sample-backup-plan --project=sample-project --location=us-central1
            --resource-type 'compute.<UNIVERSE_DOMAIN>/Instance'
            --backup-vault <BACKUP-VAULT>
            --backup-rule <BACKUP-RULE>
            --backup-rule <BACKUP-RULE>

        To create a new backup plan ``sample-filestore-backup-plan'' for Filestore instances
        in project ``sample-project'',
        at location ``us-central1'',
        with resource-type ``file.<UNIVERSE_DOMAIN>/Instance'' and
        backup-vault ``backup-vault''
        with 1 backup-rule:

        run:

          $ {command} sample-filestore-backup-plan --project=sample-project --location=us-central1
            --resource-type 'file.<UNIVERSE_DOMAIN>/Instance'
            --backup-vault <BACKUP-VAULT>
            --backup-rule <BACKUP-RULE>

        Backup Rule Examples:

        1. Hourly backup rule with hourly backup frequency of 6 hours and store it for 30 days, and expect the backups to run only between 10:00 to 20:00 UTC

        <BACKUP-RULE>: rule-id=sample-hourly-rule,retention-days=30,recurrence=HOURLY,hourly-frequency=6,time-zone=UTC,backup-window-start=10,backup-window-end=20

        Properties:
          -- rule-id = "sample-hourly-rule"
          -- retention-days = 30
          -- recurrence = HOURLY
          -- hourly-frequency = 6
          -- time-zone = UTC
          -- backup-window-start = 10
          -- backup-window-end = 20

        2. Daily backup rule with daily backup frequency of 6 hours and store it for 7 days

        <BACKUP-RULE>: rule-id=sample-daily-rule,retention-days=7,recurrence=DAILY,backup-window-start=1,backup-window-end=14

        Properties:
          -- rule-id = "sample-daily-rule"
          -- retention-days = 7
          -- recurrence = DAILY
          -- backup-window-start = 1
          -- backup-window-end = 14

        3. Weekly backup rule with weekly backup frequency on every MONDAY & FRIDAY and store it for 21 days

        <BACKUP-RULE>: rule-id=sample-weekly-rule,retention-days=21,recurrence=WEEKLY,days-of-week="MONDAY FRIDAY",backup-window-start=10,backup-window-end=20

        Properties:
          -- rule-id = "sample-weekly-rule"
          -- retention-days: 21
          -- recurrence = WEEKLY
          -- days-of-week = "MONDAY FRIDAY"
          -- backup-window-start = 10
          -- backup-window-end = 20
        """,
  }

  @staticmethod
  def Args(parser):
    """Specifies additional command flags.

    Args:
      parser: argparse.Parser: Parser object for command line inputs.
    """
    base.ASYNC_FLAG.AddToParser(parser)
    base.ASYNC_FLAG.SetDefault(parser, True)
    flags.AddBackupPlanResourceArgWithBackupVault(
        parser,
        """Name of the backup plan to be created.
        Once the backup plan is created, this name can't be changed.
        The name must be unique for a project and location.
        """,
    )
    flags.AddResourceType(
        parser,
        textwrap.dedent("""\
        Type of resource to which the backup plan should be applied.

        For example:
        * `compute.<UNIVERSE_DOMAIN>/Instance` for Compute Engine instances.
        * `file.<UNIVERSE_DOMAIN>/Instance` for Filestore instances.""")
    )
    flags.AddBackupRule(parser)
    flags.AddMaxCustomOnDemandRetentionDays(parser)
    flags.AddDiskBackupPlanProperties(parser)
    flags.AddComputeInstanceBackupPlanProperties(parser)

    description_help = """\
        Provide a description of the backup plan, such as specific use cases and
        relevant details, in 2048 characters or less.

        E.g., This is a backup plan that performs a daily backup at 6 p.m. and
        retains data for 3 months.
        """
    flags.AddDescription(parser, description_help)

    labels_help = """\
        If you have assigned labels to your resources for grouping, you can
        provide the label using this flag.A label is a key-value pair.

        Keys must start with a lowercase character and contain only hyphens (-),
        underscores (_), lowercase characters, and numbers. Values must contain
        only hyphens (-), underscores (_), lowercase characters, and numbers.
        """
    flags.AddLabels(parser, labels_help)
    flags.AddLogRetentionDays(parser)

  def Run(self, args):
    """Constructs and sends request.

    Args:
      args: argparse.Namespace, An object that contains the values for the
        arguments specified in the .Args() method.

    Returns:
      ProcessHttpResponse of the request made.
    """
    api_version = util.GetApiVersion(self.ReleaseTrack())
    client = backup_plans.BackupPlansClient(api_version=api_version)
    is_async = args.async_

    backup_plan = args.CONCEPTS.backup_plan.Parse()
    backup_vault = args.CONCEPTS.backup_vault.Parse()
    resource_type = args.resource_type
    backup_rules = args.backup_rule
    log_retention_days = args.log_retention_days
    description = args.description
    labels = args.labels
    max_custom_on_demand_retention_days = (
        args.max_custom_on_demand_retention_days
    )
    disk_properties = args.disk_properties
    compute_instance_properties = args.compute_instance_properties

    try:
      operation = client.Create(
          backup_plan,
          backup_vault.RelativeName(),
          resource_type,
          backup_rules,
          log_retention_days,
          description,
          labels,
          max_custom_on_demand_retention_days,
          disk_properties,
          compute_instance_properties,
      )
    except apitools_exceptions.HttpError as e:
      raise exceptions.HttpException(e, util.HTTP_ERROR_FORMAT)

    if is_async:
      log.CreatedResource(
          backup_plan.RelativeName(),
          kind='backup plan',
          is_async=True,
          details=util.ASYNC_OPERATION_MESSAGE.format(operation.name),
      )
      return operation

    resource = client.WaitForOperation(
        operation_ref=client.GetOperationRef(operation),
        message=(
            'Creating backup plan [{}]. (This operation could'
            ' take up to 2 minutes.)'.format(backup_plan.RelativeName())
        ),
    )
    log.CreatedResource(backup_plan.RelativeName(), kind='backup plan')

    return resource



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Lists backup configurations for all volumes in an ONTAP-mode Storage Pool."""

import argparse
import textwrap
from googlecloudsdk.api_lib.netapp.storage_pools import client as storagepools_client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.netapp import flags
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.core import properties


@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.GA)
class ListBackupConfigs(base.ListCommand):
  """List backup configurations for all volumes in an ONTAP-mode Storage Pool."""

  _RELEASE_TRACK = base.ReleaseTrack.GA

  detailed_help = {
      'DESCRIPTION': textwrap.dedent("""\
          Lists backup configurations for all volumes in an ONTAP-mode Storage Pool.
          """),
      'EXAMPLES': textwrap.dedent("""\
          The following command lists backup configurations in Storage Pool named NAME in location us-central1:

              $ {command} NAME --location=us-central1
          """),
  }

  @staticmethod
  def Args(parser: argparse.ArgumentParser) -> None:
    concept_parsers.ConceptParser([
        flags.GetStoragePoolPresentationSpec(
            'The Storage Pool to list backup configs for.')
    ]).AddToParser(parser)

  def Run(self, args):
    """Run the list-backup-configs command."""
    properties.VALUES.core.project.GetOrFail()
    storagepool_ref = args.CONCEPTS.storage_pool.Parse()
    client = storagepools_client.StoragePoolsClient(
        release_track=self._RELEASE_TRACK)
    return client.ListBackupConfigs(storagepool_ref, limit=args.limit)


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class ListBackupConfigsBeta(ListBackupConfigs):
  """List backup configurations for all volumes in an ONTAP-mode Storage Pool."""

  _RELEASE_TRACK = base.ReleaseTrack.BETA


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class ListBackupConfigsAlpha(ListBackupConfigsBeta):
  """List backup configurations for all volumes in an ONTAP-mode Storage Pool."""

  _RELEASE_TRACK = base.ReleaseTrack.ALPHA



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Cloud Datastream API utilities."""


import uuid

from apitools.base.py import encoding as api_encoding
from googlecloudsdk.api_lib.dataproc import exceptions
from googlecloudsdk.api_lib.datastream import camel_case_utils
from googlecloudsdk.api_lib.datastream import exceptions as ds_exceptions
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.export import util as export_util
from googlecloudsdk.core import resources
from googlecloudsdk.core import yaml
from googlecloudsdk.core.console import console_io
import six


_DEFAULT_API_VERSION = 'v1'
_DEFAULT_API_NAME = 'datastream'
CAMEL_CASE_CONVERSION_EVENT = _DEFAULT_API_NAME + '_camel_case_conversion'

# TODO(b/207467120): remove translation after BETA deprecation.
_UPDATE_MASK_FIELD_TRANSLATION_V1ALPHA1_TO_V1 = {
    'allowlist': 'include_objects',
    'rejectlist': 'exclude_objects',
    'source_connection_profile_name': 'source_connection_profile',
    'destination_connection_profile_name': 'destination_connection_profile',
    'vpc_name': 'vpc',
}
RDBMS_FIELD_NAME_BY_RELEASE_TRACK = {
    'schema': {
        base.ReleaseTrack.BETA: 'schema_name',
        base.ReleaseTrack.GA: 'schema'
    },
    'database': {
        base.ReleaseTrack.BETA: 'database_name',
        base.ReleaseTrack.GA: 'database'
    },
    'table': {
        base.ReleaseTrack.BETA: 'table_name',
        base.ReleaseTrack.GA: 'table'
    },
    'column': {
        base.ReleaseTrack.BETA: 'column_name',
        base.ReleaseTrack.GA: 'column'
    },
    'include_objects': {
        base.ReleaseTrack.BETA: 'allowlist',
        base.ReleaseTrack.GA: 'include_objects'
    },
    'exclude_objects': {
        base.ReleaseTrack.BETA: 'rejectlist',
        base.ReleaseTrack.GA: 'exclude_objects'
    }
}


def ParseJsonAndValidateSchema(parsed_json, schema_name, message_type):
  """Parses a config message from a parsed JSON and validates its schema."""
  schema_path = export_util.GetSchemaPath(
      _DEFAULT_API_NAME, _DEFAULT_API_VERSION, schema_name, for_help=False
  )

  message = CreateMessageWithCamelCaseConversion(
      message_type=message_type,
      parsed_yaml=parsed_json,
      schema_path=schema_path,
  )

  return message


def ParseMessageAndValidateSchema(config_file_path, schema_name, message_type):
  """Parses a config message from a file and validates its schema."""
  schema_path = export_util.GetSchemaPath(
      _DEFAULT_API_NAME, _DEFAULT_API_VERSION, schema_name, for_help=False
  )

  # NOMUTANTS -- not necessary here.
  data = console_io.ReadFromFileOrStdin(config_file_path, binary=False)
  parsed_yaml = yaml.load(data)

  message = CreateMessageWithCamelCaseConversion(
      message_type=message_type,
      parsed_yaml=parsed_yaml,
      schema_path=schema_path,
  )

  return message


def GetClientInstance(
    api_version=_DEFAULT_API_VERSION, no_http=False, location=None
):
  return apis.GetClientInstance(
      'datastream', api_version, no_http=no_http, location=location
  )


def GetMessagesModule(api_version=_DEFAULT_API_VERSION):
  return apis.GetMessagesModule('datastream', api_version)


def GetResourceParser(api_version=_DEFAULT_API_VERSION):
  resource_parser = resources.Registry()
  resource_parser.RegisterApiByName('datastream', api_version)
  return resource_parser


def ParentRef(project, location):
  """Get the resource name of the parent collection.

  Args:
    project: the project of the parent collection.
    location: the GCP region of the membership.

  Returns:
    the resource name of the parent collection in the format of
    `projects/{project}/locations/{location}`.
  """

  return 'projects/{}/locations/{}'.format(project, location)


def GenerateRequestId():
  """Generates a UUID to use as the request ID.

  Returns:
    string, the 40-character UUID for the request ID.
  """
  return six.text_type(uuid.uuid4())


def ParseMysqlRdbmsFile(
    messages, mysql_rdbms_file, release_track=base.ReleaseTrack.BETA
):
  """Parses a mysql_rdbms_file into the MysqlRdbms message."""
  if release_track != base.ReleaseTrack.BETA:
    return ParseMessageAndValidateSchema(
        mysql_rdbms_file, 'MysqlRdbms', messages.MysqlRdbms
    )

  return ParseMysqlRdbmsFileBeta(messages, mysql_rdbms_file, release_track)


def ParseOracleRdbmsFile(
    messages, oracle_rdbms_file, release_track=base.ReleaseTrack.BETA
):
  """Parses a oracle_rdbms_file into the OracleRdbms message."""
  if release_track != base.ReleaseTrack.BETA:
    return ParseMessageAndValidateSchema(
        oracle_rdbms_file, 'OracleRdbms', messages.OracleRdbms
    )

  return ParseOracleRdbmsFileBeta(messages, oracle_rdbms_file, release_track)


def ParsePostgresqlRdbmsFile(messages, postgresql_rdbms_file):
  """Parses a postgresql_rdbms_file into the PostgresqlRdbms message."""
  return ParseMessageAndValidateSchema(
      postgresql_rdbms_file, 'PostgresqlRdbms', messages.PostgresqlRdbms
  )


def ParseSqlServerRdbmsFile(messages, sqlserver_rdbms_file):
  """Parses a sqlserver_rdbms_file into the SqlServerRdbms message."""
  return ParseMessageAndValidateSchema(
      sqlserver_rdbms_file, 'SqlServerRdbms', messages.SqlServerRdbms
  )


def ParseSalesforceOrgFile(messages, salesforce_org_file):
  """Parses a salesforce_org_file into the SalesforceOrg message."""
  return ParseMessageAndValidateSchema(
      salesforce_org_file, 'SalesforceOrg', messages.SalesforceOrg
  )


def ParseSpannerDatabaseFile(messages, spanner_database_file):
  """Parses a spanner_rdbms_file into the SpannerRdbms message."""
  return ParseMessageAndValidateSchema(
      spanner_database_file, 'SpannerDatabase', messages.SpannerDatabase
  )


def ParseMongodbFile(messages, mongodb_file):
  """Parses a mongodb_file into the MongodbCluster message."""
  return ParseMessageAndValidateSchema(
      mongodb_file, 'MongodbCluster', messages.MongodbCluster
  )


def ParseSourceCatalogFile(messages, source_catalog_file):
  """Parses a source_catalog_file into the SourceCatalog message."""
  return ParseMessageAndValidateSchema(
      source_catalog_file, 'SourceCatalog', messages.SourceCatalog
  )


def CreateMessageWithCamelCaseConversion(
    message_type, parsed_yaml, schema_path=None
):
  """Create a message from a yaml dict.

  Similar to export_util.Import (since we convert to camel case before)
  Args:
    message_type: a Datastream message type to create.
    parsed_yaml: dict
    schema_path: str, path to the message schema to validate against.

  Returns:
    a Datastream message.
  """
  converted_yaml = camel_case_utils.ConvertYamlToCamelCase(parsed_yaml)
  if schema_path:
    # If a schema is provided, validate against it.
    export_util.ValidateYAML(converted_yaml, schema_path)
  try:
    message = api_encoding.PyValueToMessage(message_type, converted_yaml)
  except Exception as e:
    raise exceptions.ParseError('Cannot parse YAML: [{0}]'.format(e))
  return message


# TODO(b/207467120): deprecate BETA client.
def GetRDBMSV1alpha1ToV1FieldName(field, release_track):
  return RDBMS_FIELD_NAME_BY_RELEASE_TRACK.get(field, {}).get(
      release_track, field
  )


def _GetRDBMSFieldName(field, release_track):
  return RDBMS_FIELD_NAME_BY_RELEASE_TRACK.get(field, {}).get(
      release_track, field
  )


# TODO(b/207467120): Deprecated BETA methods - remove after full BETA
# deprecation.
def ParseMysqlColumn(messages, mysql_column_object, release_track):
  """Parses a raw mysql column json/yaml into the MysqlColumn message."""
  message = messages.MysqlColumn(
      column=mysql_column_object.get(
          _GetRDBMSFieldName('column', release_track), ''))
  data_type = mysql_column_object.get('data_type')
  if data_type is not None:
    message.dataType = data_type
  collation = mysql_column_object.get('collation')
  if collation is not None:
    message.collation = collation
  length = mysql_column_object.get('length')
  if length is not None:
    message.length = length
  nullable = mysql_column_object.get('nullable')
  if nullable is not None:
    message.nullable = nullable
  ordinal_position = mysql_column_object.get('ordinal_position')
  if ordinal_position is not None:
    message.ordinalPosition = ordinal_position
  primary_key = mysql_column_object.get('primary_key')
  if primary_key is not None:
    message.primaryKey = primary_key
  return message


def ParseMysqlTable(messages, mysql_table_object, release_track):
  """Parses a raw mysql table json/yaml into the MysqlTable message."""
  mysql_column_msg_list = []
  for column in mysql_table_object.get('mysql_columns', []):
    mysql_column_msg_list.append(
        ParseMysqlColumn(messages, column, release_track))
  table_key = _GetRDBMSFieldName('table', release_track)
  table_name = mysql_table_object.get(table_key)
  if not table_name:
    raise ds_exceptions.ParseError('Cannot parse YAML: missing key "%s".' %
                                   table_key)
  return messages.MysqlTable(
      table=table_name, mysqlColumns=mysql_column_msg_list)


def ParseMysqlDatabase(messages, mysql_database_object, release_track):
  """Parses a raw mysql database json/yaml into the MysqlDatabase message."""
  mysql_tables_msg_list = []
  for table in mysql_database_object.get('mysql_tables', []):
    mysql_tables_msg_list.append(
        ParseMysqlTable(messages, table, release_track))
  database_key = _GetRDBMSFieldName('database', release_track)
  database_name = mysql_database_object.get(database_key)
  if not database_name:
    raise ds_exceptions.ParseError('Cannot parse YAML: missing key "%s".' %
                                   database_key)
  return messages.MysqlDatabase(
      database=database_name, mysqlTables=mysql_tables_msg_list)


def ParseMysqlSchemasListToMysqlRdbmsMessage(messages,
                                             mysql_rdbms_data,
                                             release_track=base.ReleaseTrack
                                             .BETA):
  """Parses an object of type {mysql_databases: [...]} into the MysqlRdbms message."""
  mysql_databases_raw = mysql_rdbms_data.get('mysql_databases', [])
  mysql_database_msg_list = []
  for schema in mysql_databases_raw:
    mysql_database_msg_list.append(
        ParseMysqlDatabase(messages, schema, release_track))

  mysql_rdbms_msg = messages.MysqlRdbms(
      mysqlDatabases=mysql_database_msg_list)
  return mysql_rdbms_msg


def ParseOracleColumn(messages, oracle_column_object, release_track):
  """Parses a raw oracle column json/yaml into the OracleColumn message."""
  message = messages.OracleColumn(
      column=oracle_column_object.get(
          _GetRDBMSFieldName('column', release_track), ''))
  data_type = oracle_column_object.get('data_type')
  if data_type is not None:
    message.dataType = data_type
  encoding = oracle_column_object.get('encoding')
  if encoding is not None:
    message.encoding = encoding
  length = oracle_column_object.get('length')
  if length is not None:
    message.length = length
  nullable = oracle_column_object.get('nullable')
  if nullable is not None:
    message.nullable = nullable
  ordinal_position = oracle_column_object.get('ordinal_position')
  if ordinal_position is not None:
    message.ordinalPosition = ordinal_position
  precision = oracle_column_object.get('precision')
  if precision is not None:
    message.precision = precision
  primary_key = oracle_column_object.get('primary_key')
  if primary_key is not None:
    message.primaryKey = primary_key
  scale = oracle_column_object.get('scale')
  if scale is not None:
    message.scale = scale
  return message


def ParseOracleTable(messages, oracle_table_object, release_track):
  """Parses a raw oracle table json/yaml into the OracleTable message."""
  oracle_columns_msg_list = []
  for column in oracle_table_object.get('oracle_columns', []):
    oracle_columns_msg_list.append(
        ParseOracleColumn(messages, column, release_track))
  table_key = _GetRDBMSFieldName('table', release_track)
  table_name = oracle_table_object.get(table_key)
  if not table_name:
    raise ds_exceptions.ParseError('Cannot parse YAML: missing key "%s".' %
                                   table_key)
  return messages.OracleTable(
      table=table_name, oracleColumns=oracle_columns_msg_list)


def ParseOracleSchema(messages, oracle_schema_object, release_track):
  """Parses a raw oracle schema json/yaml into the OracleSchema message."""
  oracle_tables_msg_list = []
  for table in oracle_schema_object.get('oracle_tables', []):
    oracle_tables_msg_list.append(
        ParseOracleTable(messages, table, release_track))
  schema_key = _GetRDBMSFieldName('schema', release_track)
  schema_name = oracle_schema_object.get(schema_key)
  if not schema_name:
    raise ds_exceptions.ParseError('Cannot parse YAML: missing key "%s".' %
                                   schema_key)
  return messages.OracleSchema(
      schema=schema_name, oracleTables=oracle_tables_msg_list)


def ParseOracleSchemasListToOracleRdbmsMessage(messages,
                                               oracle_rdbms_data,
                                               release_track=base.ReleaseTrack
                                               .BETA):
  """Parses an object of type {oracle_schemas: [...]} into the OracleRdbms message."""
  oracle_schemas_raw = oracle_rdbms_data.get('oracle_schemas', [])
  oracle_schema_msg_list = []
  for schema in oracle_schemas_raw:
    oracle_schema_msg_list.append(
        ParseOracleSchema(messages, schema, release_track))

  oracle_rdbms_msg = messages.OracleRdbms(
      oracleSchemas=oracle_schema_msg_list)
  return oracle_rdbms_msg


def ParsePostgresqlColumn(messages, postgresql_column_object):
  """Parses a raw postgresql column json/yaml into the PostgresqlColumn message."""
  message = messages.PostgresqlColumn(
      column=postgresql_column_object.get('column', ''))
  data_type = postgresql_column_object.get('data_type')
  if data_type is not None:
    message.dataType = data_type
  length = postgresql_column_object.get('length')
  if length is not None:
    message.length = length
  precision = postgresql_column_object.get('precision')
  if precision is not None:
    message.precision = precision
  scale = postgresql_column_object.get('scale')
  if scale is not None:
    message.scale = scale
  primary_key = postgresql_column_object.get('primary_key')
  if primary_key is not None:
    message.primaryKey = primary_key
  nullable = postgresql_column_object.get('nullable')
  if nullable is not None:
    message.nullable = nullable
  ordinal_position = postgresql_column_object.get('ordinal_position')
  if ordinal_position is not None:
    message.ordinalPosition = ordinal_position
  return message


def ParsePostgresqlTable(messages, postgresql_table_object):
  """Parses a raw postgresql table json/yaml into the PostgresqlTable message."""
  postgresql_columns_msg_list = []
  for column in postgresql_table_object.get('postgresql_columns', []):
    postgresql_columns_msg_list.append(ParsePostgresqlColumn(messages, column))
  table_name = postgresql_table_object.get('table')
  if not table_name:
    raise ds_exceptions.ParseError('Cannot parse YAML: missing key "table".')
  return messages.PostgresqlTable(
      table=table_name, postgresqlColumns=postgresql_columns_msg_list)


def ParsePostgresqlSchema(messages, postgresql_schema_object):
  """Parses a raw postgresql schema json/yaml into the PostgresqlSchema message."""
  postgresql_tables_msg_list = []
  for table in postgresql_schema_object.get('postgresql_tables', []):
    postgresql_tables_msg_list.append(ParsePostgresqlTable(messages, table))
  schema_name = postgresql_schema_object.get('schema')
  if not schema_name:
    raise ds_exceptions.ParseError('Cannot parse YAML: missing key "schema".')
  return messages.PostgresqlSchema(
      schema=schema_name, postgresqlTables=postgresql_tables_msg_list)


def ParsePostgresqlSchemasListToPostgresqlRdbmsMessage(messages,
                                                       postgresql_rdbms_data):
  """Parses an object of type {postgresql_schemas: [...]} into the PostgresqlRdbms message."""
  postgresql_schemas_raw = postgresql_rdbms_data.get('postgresql_schemas', [])
  postgresql_schema_msg_list = []
  for schema in postgresql_schemas_raw:
    postgresql_schema_msg_list.append(ParsePostgresqlSchema(messages, schema))

  postgresql_rdbms_msg = messages.PostgresqlRdbms(
      postgresqlSchemas=postgresql_schema_msg_list)
  return postgresql_rdbms_msg


def ParseMongodbField(messages, mongodb_field_object):
  """Parses a raw mongodb field json/yaml into the MongodbField message."""
  return messages.MongodbField(field=mongodb_field_object.get('field', ''))


def ParseMongodbCollection(messages, mongodb_collection_object):
  """Parses a raw mongodb database json/yaml into the MongodbCollection message."""
  fields_raw = mongodb_collection_object.get('fields', [])
  fields_msg_list = []
  for field in fields_raw:
    fields_msg_list.append(ParseMongodbField(messages, field))
  return messages.MongodbCollection(
      collection=mongodb_collection_object.get('collection', ''),
      fields=fields_msg_list,
  )


def ParseMongodbDatabase(messages, mongodb_database_object):
  """Parses a raw mongodb database json/yaml into the MongodbDatabase message."""
  collections_raw = mongodb_database_object.get('collections', [])
  collections_msg_list = []
  for collection in collections_raw:
    collections_msg_list.append(ParseMongodbCollection(messages, collection))
  return messages.MongodbDatabase(collections=collections_msg_list)


def ParseMongodbCluster(messages, mongodb_cluster_object):
  """Parses a raw mongodb cluster json/yaml into the MongodbCluster message."""
  databases_raw = mongodb_cluster_object.get('databases', [])
  databases_msg_list = []
  for database in databases_raw:
    databases_msg_list.append(ParseMongodbDatabase(messages, database))
  return messages.MongodbCluster(databases=databases_msg_list)


def UpdateV1alpha1ToV1MaskFields(field_mask):
  """Updates field mask paths according to the v1alpha1 > v1 Datastream API change.

  This allows for backwards compatibility with the current client field
  mask.

  Args:
    field_mask: List[str], list of stream fields to update

  Returns:
    updated_field_mask: List[str] field mask with fields translated
      from v1alpha1 API to v1.
  """
  updated_field_mask = []
  for path in field_mask:
    field_to_translate = None
    for field in _UPDATE_MASK_FIELD_TRANSLATION_V1ALPHA1_TO_V1:
      if field in path:
        field_to_translate = field
        break
    if field_to_translate:
      updated_field_mask.append(
          path.replace(
              field_to_translate,
              _UPDATE_MASK_FIELD_TRANSLATION_V1ALPHA1_TO_V1[field_to_translate])
      )
    else:
      updated_field_mask.append(path)
  return updated_field_mask


def ParseMysqlRdbmsFileBeta(
    messages, mysql_rdbms_file, release_track=base.ReleaseTrack.BETA
):
  """Parses a mysql_rdbms_file into the MysqlRdbms message. deprecated."""

  data = console_io.ReadFromFileOrStdin(mysql_rdbms_file, binary=False)
  try:
    mysql_rdbms_head_data = yaml.load(data)
  except Exception as e:
    raise ds_exceptions.ParseError('Cannot parse YAML:[{0}]'.format(e))

  mysql_rdbms_data = mysql_rdbms_head_data.get(
      'mysql_rdbms', mysql_rdbms_head_data
  )
  return ParseMysqlSchemasListToMysqlRdbmsMessage(
      messages, mysql_rdbms_data, release_track
  )


def ParseOracleRdbmsFileBeta(
    messages, oracle_rdbms_file, release_track=base.ReleaseTrack.BETA
):
  """Parses a oracle_rdbms_file into the OracleRdbms message. deprecated."""
  data = console_io.ReadFromFileOrStdin(oracle_rdbms_file, binary=False)
  try:
    oracle_rdbms_head_data = yaml.load(data)
  except Exception as e:
    raise ds_exceptions.ParseError('Cannot parse YAML:[{0}]'.format(e))

  oracle_rdbms_data = oracle_rdbms_head_data.get(
      'oracle_rdbms', oracle_rdbms_head_data
  )
  return ParseOracleSchemasListToOracleRdbmsMessage(
      messages, oracle_rdbms_data, release_track
  )



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Repair cluster command."""


from googlecloudsdk.api_lib.dataproc import dataproc as dp
from googlecloudsdk.api_lib.dataproc import util
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.dataproc import flags
from googlecloudsdk.command_lib.util.apis import arg_utils
from googlecloudsdk.core import log
from googlecloudsdk.core.console import console_io


@base.UniverseCompatible
@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA, base.ReleaseTrack.GA
)
@base.Hidden
class Repair(base.Command):
  """Repair a cluster."""

  detailed_help = {
      'EXAMPLES': """
          To repair an ERROR_DUE_TO_UPDATE cluster back to RUNNING state, run:

            $ {command} my-cluster --region=us-central1 \
               --cluster-repair-action=REPAIR_ERROR_DUE_TO_UPDATE_CLUSTER

          To repair a cluster by deleting faulty primary worker nodes, run:

            $ {command} my-cluster --region=us-central1 \
              --node-pool id=PRIMARY_WORKER_POOL,repair-action=delete,instance-names="w-1;w-10"

          To repair a cluster by deleting faulty secondary worker nodes, run:

            $ {command} my-cluster --region=us-central1 \
              --node-pool id=SECONDARY_WORKER_POOL,repair-action=delete,instance-names="sw-1;sw-10"

          To repair a cluster by deleting faulty nodes from different pools, run:

            $ {command} my-cluster --region=us-central1 \
              --node-pool id=PRIMARY_WORKER_POOL,repair-action=delete,instance-names="w-1;w-10" \
              --node-pool id=SECONDARY_WORKER_POOL,repair-action=delete,instance-names="sw-1;sw-10"
          """,
  }

  @classmethod
  def Args(cls, parser):
    """Parse arguments for repair command."""
    dataproc = dp.Dataproc(cls.ReleaseTrack())
    base.ASYNC_FLAG.AddToParser(parser)
    flags.AddTimeoutFlag(parser)
    flags.AddClusterResourceArg(parser, 'repair', dataproc.api_version)
    parser.add_argument(
        '--dataproc-super-user',
        type=bool,
        default=False,
        help=(
            'Whether to use Dataproc superuser permissions. If true, IAM checks'
            ' for `dataproc.googleapis.com/clusters.repair` permission,'
            ' otherwise `dataproc.googleapis.com/clusters.update`'
        ),
        hidden=True,
    )
    repair_target = parser.add_argument_group(mutex=True, required=True)
    repair_target.add_argument(
        '--cluster-repair-action',
        help="""
          `--cluster-repair-action` flag indicates the repair operation is at
          the cluster level. `--node-pool` flag should not be specified with
          this flag. Valid values : {}.
          """.format(cls._GetValidClusterRepairActionChoices(dataproc)),
    )
    node_pool = repair_target.add_argument_group(help='Node pool flags')
    node_pool.add_argument(
        '--node-pool',
        type=arg_parsers.ArgDict(
            required_keys=['id', 'repair-action', 'instance-names'],
            spec={
                'id': str,
                'repair-action': cls._GetParseNodePoolRepairActionFunc(
                    dataproc
                ),
                'instance-names': arg_parsers.ArgList(custom_delim_char=';'),
            },
        ),
        action='append',
        default=[],
        metavar='id=ID,repair-action=REPAIR_ACTION,instance-names="INSTANCE_NAME1[;INSTANCE_NAME2]"',
        help="""
          Each `--node-pool` flag represents either the primary or secondary
          worker pool associated with the cluster and an action on specified
          nodes.
          *id:*::: Valid values : {}.
          *repair-action:*::: Valid values : {}.
          """.format(
            cls._GetValidNodePoolIdChoices(),
            cls._GetValidNodePoolRepairActionChoices(dataproc),
        ),
    )
    node_pool.add_argument(
        '--graceful-decommission-timeout',
        type=arg_parsers.Duration(lower_bound='0s', upper_bound='1d'),
        help="""
              The graceful decommission timeout for decommissioning Node Managers
              in the cluster, used when removing nodes. Graceful decommissioning
              allows removing nodes from the cluster without interrupting jobs in
              progress. Timeout specifies how long to wait for jobs in progress to
              finish before forcefully removing nodes (and potentially
              interrupting jobs). Timeout defaults to 0 if not set (for forceful
              decommission), and the maximum allowed timeout is 1 day.
              See $ gcloud topic datetimes for information on duration formats.
              """,
    )

  @classmethod
  def _GetParseNodePoolRepairActionFunc(cls, dataproc):
    """Get the function to verify node pool repair-action values."""

    def _ParseNodePoolRepairActionFunc(repair_action=None):
      return arg_utils.ChoiceToEnum(
          repair_action,
          dataproc.messages.NodePool.RepairActionValueValuesEnum,
          item_type='NODE_POOL_REPAIR_ACTION',
          valid_choices=cls._GetValidNodePoolRepairActionChoices(dataproc),
      )

    return _ParseNodePoolRepairActionFunc

  @classmethod
  def _GetValidNodePoolIdChoices(cls):
    """Get list of valid node-pool id values."""
    return ['PRIMARY_WORKER_POOL', 'SECONDARY_WORKER_POOL']

  @classmethod
  def _GetValidNodePoolRepairActionChoices(cls, dataproc):
    """Get list of valid REPAIR_ACTION values."""
    repair_action_enums = dataproc.messages.NodePool.RepairActionValueValuesEnum
    return [
        arg_utils.ChoiceToEnumName(n)
        for n in repair_action_enums.names()
        if n != 'REPAIR_ACTION_UNSPECIFIED'
    ]

  @classmethod
  def _GetValidClusterRepairActionChoices(cls, dataproc):
    """Get list of valid REPAIR_ACTION values."""
    repair_action_enums = (
        dataproc.messages.ClusterToRepair.ClusterRepairActionValueValuesEnum
    )
    return [
        arg_utils.ChoiceToEnumName(n)
        for n in repair_action_enums.names()
        if n != 'CLUSTER_REPAIR_ACTION_UNSPECIFIED'
    ]

  def _ParseClusterRepairAction(self, dataproc, cluster_repair_action):
    """Get the function to verify cluster repair-action values."""
    return arg_utils.ChoiceToEnum(
        cluster_repair_action,
        dataproc.messages.ClusterToRepair.ClusterRepairActionValueValuesEnum,
        item_type='CLUSTER_REPAIR_ACTION',
        valid_choices=self._GetValidClusterRepairActionChoices(dataproc),
    )

  def _ParseNodePool(self, dataproc, node_pool):
    """Parses a single --node-pool flag into a NodePool message."""
    return dataproc.messages.NodePool(
        id=node_pool['id'],
        repairAction=node_pool['repair-action'],
        instanceNames=node_pool['instance-names'])

  def _ParseNodePools(self, dataproc, args_node_pools):
    """Parses all --node-pool flags into a list of NodePool messages."""
    pools = [
        self._ParseNodePool(dataproc, node_pool)
        for node_pool in args_node_pools
    ]
    self._ValidateNodePoolIds(pools)
    return pools

  def _ValidateNodePoolIds(self, node_pools):
    """Validates whether node-pools are valid."""
    valid_ids = self._GetValidNodePoolIdChoices()
    for node_pool in node_pools:
      node_pool_id = node_pool.id
      if node_pool_id not in valid_ids:
        raise exceptions.InvalidArgumentException(
            '--node-pool',
            'Node pool ID "{}" is not one of {}'.format(node_pool_id,
                                                        valid_ids))

    unique_ids = set()
    for node_pool in node_pools:
      node_pool_id = node_pool.id
      if node_pool_id in unique_ids:
        raise exceptions.InvalidArgumentException(
            '--node-pool',
            'Node pool id "{}" used more than once.'.format(node_pool_id))
      unique_ids.add(node_pool_id)

  def Run(self, args):
    dataproc = dp.Dataproc(self.ReleaseTrack())
    cluster_ref = args.CONCEPTS.cluster.Parse()

    repair_cluster_request = None
    if args.node_pool:
      repair_cluster_request = dataproc.messages.RepairClusterRequest(
          requestId=util.GetUniqueId(),
          nodePools=self._ParseNodePools(dataproc, args.node_pool),
          dataprocSuperUser=args.dataproc_super_user,
      )

      if args.graceful_decommission_timeout is not None:
        repair_cluster_request.gracefulDecommissionTimeout = (
            str(args.graceful_decommission_timeout) + 's'
        )

      console_io.PromptContinue(
          message=(
              "The specified nodes in cluster '{0}' and all"
              ' attached disks will be deleted.'.format(cluster_ref.clusterName)
          ),
          cancel_on_no=True,
          cancel_string='Repair canceled by user.',
      )

    if args.cluster_repair_action:
      repair_cluster_request = dataproc.messages.RepairClusterRequest(
          requestId=util.GetUniqueId(),
          cluster=dataproc.messages.ClusterToRepair(
              clusterRepairAction=self._ParseClusterRepairAction(
                  dataproc, args.cluster_repair_action
              )
          ),
          dataprocSuperUser=args.dataproc_super_user,
      )

    request = dataproc.messages.DataprocProjectsRegionsClustersRepairRequest(
        clusterName=cluster_ref.clusterName,
        region=cluster_ref.region,
        projectId=cluster_ref.projectId,
        repairClusterRequest=repair_cluster_request,
    )
    operation = dataproc.client.projects_regions_clusters.Repair(request)

    if args.async_:
      log.status.write('Repairing [{0}] with operation [{1}].'.format(
          cluster_ref, operation.name))
      return operation

    return util.WaitForOperation(
        dataproc,
        operation,
        message="Waiting for cluster '{0}' repair to finish.".format(
            cluster_ref.clusterName),
        timeout_s=args.timeout)



# -*- coding: utf-8 -*-
# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""Aiplatform gRPC client. This class is automatically generated."""

import contextlib

from googlecloudsdk.core import gapic_util
from googlecloudsdk.generated_clients.gapic_clients import aiplatform_v1


class GapicWrapperClient(object):
  """Aiplatform client."""
  types = aiplatform_v1.types

  def __init__(self, credentials, **kwargs):
    """
    Instantiates the GapicWrapperClient for aiplatform_v1.

    Args:
      credentials: google.auth.credentials.Credentials, the credentials to use.
      **kwargs: Additional kwargs to pass to gapic.MakeClient.

    Returns:
        GapicWrapperClient
    """
    self._exit_stack = contextlib.ExitStack()
    self.credentials = credentials
    self.prediction = gapic_util.MakeClient(
        aiplatform_v1.services.prediction_service.client.PredictionServiceClient,
        credentials, **kwargs)
    self.semantic = gapic_util.MakeClient(
        aiplatform_v1.services.semantic_governance_policy_engine_service.client.SemanticGovernancePolicyEngineServiceClient,
        credentials, **kwargs)
    self.semantic = gapic_util.MakeClient(
        aiplatform_v1.services.semantic_governance_policy_service.client.SemanticGovernancePolicyServiceClient,
        credentials, **kwargs)

  def __enter__(self):
    self._exit_stack.enter_context(self.prediction)
    self._exit_stack.enter_context(self.semantic)
    self._exit_stack.enter_context(self.semantic)
    return self

  def close(self):
    if hasattr(self, '_exit_stack'):
      self._exit_stack.close()

  def __exit__(self, exc_type, exc_value, traceback):
    self.close()

  def __del__(self):
    try:
      self.close()
    except Exception:
      pass



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The create command for BigLake Iceberg REST catalogs."""

import textwrap

from googlecloudsdk.api_lib.biglake import util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.biglake import arguments
from googlecloudsdk.command_lib.biglake import flags
from googlecloudsdk.core import log
from googlecloudsdk.core.util import times

help_text = textwrap.dedent("""\
    To add a catalog using a Cloud Storage bucket `my-catalog-bucket`, run:

      $ {command} my-catalog-bucket --catalog-type=gcs-bucket

    To create a catalog using a Cloud Storage bucket `my-catalog-bucket` with vended credentials, run:

      $ {command} my-catalog-bucket --catalog-type=gcs-bucket --credential-mode=vended-credentials
    """)

help_text_preview = textwrap.dedent("""\
    To create a unity federated catalog `my-federated-catalog`, run:

      $ {command} my-federated-catalog --catalog-type=federated --federated-catalog-type=unity --service-directory-name=projects/my-project/locations/us/namespaces/my-namespace/services/my-service --unity-catalog-name=my-catalog --unity-instance-name=instance.cloud.databricks.com --unity-service-principal-application-id=xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx --primary-location=us
    """)


def _BuildFederatedCatalogMessage(args, messages):
  """Builds FederatedCatalogMessage for federated catalogs."""
  if args.federated_catalog_type == 'unity':
    federated_catalog_options = messages.FederatedCatalogOptions(
        service_directory_name=getattr(args, 'service_directory_name', None),
        secret_name=args.secret_name,
        unity_catalog_info=messages.UnityCatalogInfo(
            catalog_name=args.unity_catalog_name,
            instance_name=args.unity_instance_name,
            service_principal_application_id=getattr(
                args, 'unity_service_principal_application_id', None
            ),
        ),
    )
  elif args.federated_catalog_type == 'glue':
    federated_catalog_options = messages.FederatedCatalogOptions(
        service_directory_name=getattr(args, 'service_directory_name', None),
        glue_catalog_info=messages.GlueCatalogInfo(
            warehouse=args.glue_warehouse,
            aws_region=args.glue_aws_region,
            aws_role_arn=args.glue_aws_role_arn,
        ),
    )
  elif args.federated_catalog_type == 'snowflake':
    federated_catalog_options = messages.FederatedCatalogOptions(
        secret_name=args.secret_name,
        service_directory_name=args.service_directory_name,
        snowflake_catalog_info=messages.SnowflakeCatalogInfo(
            warehouse=args.snowflake_warehouse,
            account_identifier=args.snowflake_account_identifier,
        ),
    )
  elif args.federated_catalog_type == 'workday':
    federated_catalog_options = messages.FederatedCatalogOptions(
        secret_name=args.secret_name,
        service_directory_name=args.service_directory_name,
        workday_catalog_info=messages.WorkdayCatalogInfo(
            base_url=args.workday_base_url,
            tenant=args.workday_tenant,
        ),
    )
  else:
    federated_catalog_options = messages.FederatedCatalogOptions()
  # Refresh options are supported for all federated catalog types.
  refresh_options = messages.RefreshOptions()
  if args.refresh_interval:
    refresh_options.refresh_schedule = messages.RefreshSchedule(
        refresh_interval=times.FormatDurationForJson(
            times.ParseDuration(str(args.refresh_interval) + 's')
        )
    )
  if args.namespace_filters:
    refresh_options.refresh_scope = messages.RefreshScope(
        namespace_filters=args.namespace_filters
    )
  federated_catalog_options.refresh_options = refresh_options
  return federated_catalog_options


@base.ReleaseTracks(base.ReleaseTrack.GA)
@base.DefaultUniverseOnly
class CreateCatalog(base.CreateCommand):
  """Create a BigLake Iceberg REST catalog."""

  detailed_help = {
      'EXAMPLES': help_text,
  }
  # Not supported in GA yet.
  _support_service_directory_name = False
  _support_federated_catalog = False
  _support_glue_catalog = False
  _support_snowflake_catalog = False
  _support_workday_catalog = False
  _support_unity_service_principal_application_id = False

  @classmethod
  def Args(cls, parser):
    flags.AddCatalogResourceArg(parser, 'to create')
    util.GetCredentialModeEnumMapper(
        cls.ReleaseTrack()
    ).choice_arg.AddToParser(parser)
    util.GetCatalogTypeEnumMapper(
        cls.ReleaseTrack()
    ).choice_arg.AddToParser(parser)
    arguments.AddCatalogsCreateArgs(parser)
    if cls._support_federated_catalog:
      arguments.AddFederatedCatalogArgs(
          parser,
          support_snowflake=cls._support_snowflake_catalog,
          support_workday=cls._support_workday_catalog,
      )
    if cls._support_glue_catalog:
      arguments.AddGlueCatalogArgs(parser)
    if cls._support_snowflake_catalog:
      arguments.AddSnowflakeCatalogArgs(parser)
    if cls._support_workday_catalog:
      arguments.AddWorkdayCatalogArgs(parser)
    if cls._support_service_directory_name:
      arguments.AddServiceDirectoryNameArg(parser)
    if cls._support_unity_service_principal_application_id:
      arguments.AddUnityServicePrincipalApplicationIdArg(parser)

  def Run(self, args):
    util.CheckValidArgCombinations(args)
    if self._support_federated_catalog:
      util.CheckValidFederatedArgCombinations(args)
    client = util.GetClientInstance(self.ReleaseTrack())
    messages = client.MESSAGES_MODULE

    catalog_name = util.GetCatalogName(args.catalog)

    credential_mode = None
    if args.IsSpecified('credential_mode'):
      credential_mode = util.GetCredentialModeEnumMapper(
          self.ReleaseTrack()
      ).GetEnumForChoice(args.credential_mode)

    catalog = messages.IcebergCatalog(
        name=catalog_name,
        catalog_type=util.GetCatalogTypeEnumMapper(
            self.ReleaseTrack()
        ).GetEnumForChoice(args.catalog_type),
        credential_mode=credential_mode,
    )
    if args.IsSpecified('description'):
      catalog.description = args.description
    if args.IsSpecified('default_location'):
      catalog.default_location = args.default_location
    if args.IsSpecified('restricted_locations'):
      catalog.restricted_locations_config = (
          messages.RestrictedLocationsConfig(
              restricted_locations=args.restricted_locations
          )
      )

    if self._support_federated_catalog and args.catalog_type == 'federated':
      catalog.federated_catalog_options = _BuildFederatedCatalogMessage(
          args, messages
      )

    response = util.CreateCatalog(
        args.catalog,
        catalog,
        primary_location=(
            args.primary_location
        ),
    )
    if response:
      log.CreatedResource(catalog_name, 'catalog')
      if response.biglake_service_account:
        log.status.Print(
            'BigLake service account: {}'.format(
                response.biglake_service_account
            )
        )
      if response.biglake_service_account_id:
        log.status.Print(
            'BigLake service account ID: {}'.format(
                response.biglake_service_account_id
            )
        )
    return response


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class CreateBeta(CreateCatalog):
  """Create a BigLake Iceberg REST catalog."""
  detailed_help = {
      'EXAMPLES': help_text + '\n\n' + help_text_preview,
  }
  _support_federated_catalog = True
  _support_unity_service_principal_application_id = True
  _support_glue_catalog = True


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class CreateAlpha(CreateBeta):
  """Create a BigLake Iceberg REST catalog."""

  _support_service_directory_name = True
  _support_unity_service_principal_application_id = True
  _support_glue_catalog = True
  _support_snowflake_catalog = True
  _support_workday_catalog = True



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# NOTE: This file is autogenerated and should not be edited by hand.
# AUTOGEN_CLI_VERSION: HEAD
"""Manage resources associated with Gemini Code Assist and Gemini Cloud Assist."""

from googlecloudsdk.calliope import base
from surface.gemini import _init_extensions as extensions


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@base.Autogenerated
@base.Hidden
class GeminiAlpha(extensions.GeminiAlpha):
  """Manage code repository index resources."""


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class GeminiBeta(extensions.GeminiBeta):
  """Manage resources associated with Gemini Code Assist and Gemini Cloud Assist.

  gcloud gemini commands are not associated with
  [Gemini CLI](https://github.com/google-gemini/gemini-cli).
  """


@base.ReleaseTracks(base.ReleaseTrack.GA)
@base.Autogenerated
class GeminiGa(extensions.GeminiGa):
  """Manage resources associated with Gemini Code Assist and Gemini Cloud Assist.

  gcloud gemini commands are not associated with
  [Gemini CLI](https://github.com/google-gemini/gemini-cli).
  """



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The official command to update the Config Management Feature."""

import copy
import textwrap

from apitools.base.py import exceptions as apitools_exceptions
from googlecloudsdk.api_lib.container.fleet import util
from googlecloudsdk.api_lib.util import exceptions as api_lib_exceptions
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.container.fleet import resources
from googlecloudsdk.command_lib.container.fleet.config_management import flags
from googlecloudsdk.command_lib.container.fleet.config_management import utils
from googlecloudsdk.command_lib.container.fleet.features import base as features_base
from googlecloudsdk.command_lib.container.fleet.features import flags as feature_flags
from googlecloudsdk.command_lib.container.fleet.membershipfeatures import base as mf_base
from googlecloudsdk.core import exceptions
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core.console import console_io


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class Update(mf_base.UpdateCommand, features_base.UpdateCommand):
  """Update the Config Management feature."""

  detailed_help = {
      'DESCRIPTION': textwrap.dedent("""\
          {description}

          `{command}` replaces the `apply` and `upgrade` commands for feature
          updates. The `update` command streamlines partial configuration
          changes by accepting the `describe` command output as input. In
          addition to the plural `--memberships` and `--all-memberships` flags,
          the command also supports a `--version` flag dedicated to upgrades
          that operates independently of the `--config` flag. This command
          errors if the feature does not exist."""),
      # TODO: b/435530306 - Use GA describe command in example instead.
      'EXAMPLES': textwrap.dedent(
          """\
          To update only the Config Sync sync directory on an existing
          membership configuration, run:

            $ gcloud alpha container fleet config-management describe --view=config --memberships=example-membership-1 > config.yaml

            $ sed -i "s/policyDir: foo/policyDir: bar/g" config.yaml

            $ {command} --config=config.yaml --memberships=example-membership-1

          To update the Config Sync version for all memberships to
          ``${VERSION}'', run:

            $ {command} --version="${VERSION}" --all-memberships

          To update the [fleet-default membership configuration](https://docs.cloud.google.com/kubernetes-engine/fleet-management/docs/manage-features)
          and sync select membership configurations to it, run:

            $ {command} --fleet-default-member-config=config.yaml

            $ {command} --origin=fleet --memberships=example-membership-1,example-membership-2"""
      ),
  }
  mf_name = utils.CONFIG_MANAGEMENT_FEATURE_NAME
  feature_name = utils.CONFIG_MANAGEMENT_FEATURE_NAME

  @classmethod
  def Args(cls, parser):
    all_flags_group = parser.add_group(
        required=True,
        mutex=True,
    )
    v1_api_version = util.VERSION_MAP[cls.ReleaseTrack()]
    all_flags_group.add_argument(
        '--fleet-default-member-config',
        # TODO: b/435530306 - Use GA describe command in example instead.
        help=textwrap.dedent(
            f"""\
            Path to YAML file, or `-` to read from stdin, that specifies the
            [fleet-default membership configuration](https://docs.cloud.google.com/kubernetes-engine/fleet-management/docs/manage-features)
            to update the feature to. Accepts the same schema as the
            `MembershipSpec`
            [API field](https://docs.cloud.google.com/kubernetes-engine/fleet-management/docs/reference/rpc/google.cloud.gkehub.configmanagement.{v1_api_version}#google.cloud.gkehub.configmanagement.{v1_api_version}.MembershipSpec).
            Provides the additional field-handling documented at
            https://docs.cloud.google.com/kubernetes-engine/config-sync/docs/reference/gcloud-configuration-field-behavior.

            To update only the Config Sync sync directory on the existing
            fleet-default membership configuration, run:

              $ gcloud alpha container fleet config-management describe --view=fleet-default-member-config > config.yaml

              $ sed -i "s/policyDir: foo/policyDir: bar/g" config.yaml

              $ {{command}} --fleet-default-member-config=config.yaml

            To achieve the same result in a single invocation, run:

              $ gcloud alpha container fleet config-management describe --view=fleet-default-member-config | sed "s/policyDir: foo/policyDir: bar/g" | {{command}} --fleet-default-member-config="-\""""
        ),
    )
    membership_specific_group = all_flags_group.add_group()
    configuration_group = membership_specific_group.add_group(
        required=True,
        mutex=True,
        help=(
            '`MEMBERSHIP CONFIGURATION FLAGS`. Updates membership'
            ' configuration(s), but does not wait for the resulting on-cluster'
            ' changes to complete.'
        ),
    )
    cls.ORIGIN_FLAG.AddToParser(configuration_group)
    v2_api_version = util.V2_VERSION_MAP[cls.ReleaseTrack()]
    # TODO: b/435530306 - Use GA describe command in example instead.
    configuration_group.add_argument(
        '--config',
        help=textwrap.dedent(
            f"""\
            Path to YAML file, or `-` to read from stdin, that specifies the
            configuration to update the target membership(s) to. Accepts the
            same schema as the `Spec`
            [API field](https://docs.cloud.google.com/kubernetes-engine/fleet-management/docs/reference/rpc/google.cloud.gkehub.configmanagement.{v2_api_version}#google.cloud.gkehub.configmanagement.{v2_api_version}.Spec).
            Provides the additional field-handling documented at
            https://docs.cloud.google.com/kubernetes-engine/config-sync/docs/reference/gcloud-configuration-field-behavior.

            To update the entire configuration for select memberships to that
            specified in a `config.yaml`, run:

              $ {{command}} --config=path/to/config.yaml --memberships=example-membership-1,example-membership-2

            To update only the Config Sync sync directory on an existing
            membership configuration in a single invocation, making sure to
            first inspect the existing membership configuration, run:

              $ gcloud alpha container fleet config-management describe --view=config --memberships=example-membership-1

              $ gcloud alpha container fleet config-management describe --view=config --memberships=example-membership-1 | sed "s/policyDir: foo/policyDir: bar/g" | {{command}} --config="-" --memberships=example-membership-1"""
        ),
    )
    # This group is to distinguish the functional difference between the
    # --version flag and the --config, --origin flags. It also moves the
    # per-field explanation out of the --version help text and makes room for
    # future per-field flags.
    per_field_group = configuration_group.add_group(help=textwrap.dedent(f"""\
            `PER-FIELD FLAGS`. Updates a single field and preserves all other
            existing configuration fields for target membership(s). Use the
            `--config` flag to update other configuration fields.

            Warns if the `configSync` field is not enabled. Only if the
            `configSync` field is enabled according to the `configSync.enabled`
            [field description](https://docs.cloud.google.com/kubernetes-engine/fleet-management/docs/reference/rpc/google.cloud.gkehub.configmanagement.{v2_api_version}#google.cloud.gkehub.configmanagement.{v2_api_version}.ConfigSync)
            does the feature install and manage Config Sync based on the
            membership configuration field values. Use `--verbosity=error` to
            suppress the warning.
            """))
    per_field_group.add_argument(
        '--version',
        help=textwrap.dedent(f"""\
            Version of Config Sync to update the target membership
            configuration(s) to. Upgrades Config Sync if the `configSync` field
            is enabled, and logs a warning otherwise. See the `PER-FIELD FLAGS`
            section above for details.
            To bypass the upgrade confirmation prompt, use `--quiet`.
            Find supported versions and the default version behavior in the
            `version`
            [field description](https://docs.cloud.google.com/kubernetes-engine/fleet-management/docs/reference/rpc/google.cloud.gkehub.configmanagement.{v2_api_version}#spec).

            To upgrade all memberships to the latest version, unless the
            `configSync` field is not enabled for any membership, for which the
            flag will log a warning, run:

              $ {{command}} --version="" --all-memberships
            """),
    )
    memberships_group = membership_specific_group.add_mutually_exclusive_group()
    resources.AddMembershipResourceArg(memberships_group, plural=True)
    feature_flags.ALL_MEMBERSHIPS_FLAG.AddToParser(memberships_group)

  @api_lib_exceptions.CatchHTTPErrorRaiseHTTPException('{message}')
  def Run(self, args):
    flag_parser = flags.Parser(self)
    # Empty string counts as specifying the --fleet-default-member-config flag.
    if args.fleet_default_member_config is not None:
      self.Update(
          ['fleet_default_member_config'],
          self.messages.Feature(
              fleetDefaultMemberConfig=(
                  self.messages.CommonFleetDefaultMemberConfigSpec(
                      configmanagement=flag_parser.parse_config(
                          args.fleet_default_member_config,
                          is_fleet_default=True,
                      )
                  )
              )
          ),
      )
      return
    # Do not auto-select. --version needs to search because it prompts users
    # about Config Sync versions on Memberships.
    memberships = features_base.ParseMembershipsPlural(
        args,
        prompt=True,
        search=args.version is not None,
    )
    if args.origin:
      self.sync_memberships_to_fleet_default(memberships)
      return
    # Feature must exist to reconcile v2 configuration changes.
    feature = self.GetFeature()
    if args.config is not None:
      cm_spec = flag_parser.parse_config(args.config, is_fleet_default=False)
      cm_specs_to_update = [copy.deepcopy(cm_spec) for _ in memberships]
      # Backfill even if user mapped version to empty string to avoid edge case
      # behavior and because version preservation is much safer. Backfill will
      # return the empty string anyway if user is starting clean. Users with
      # existing installations should always explicitly specify a version
      # change, including to the latest version.
      if not cm_spec.version:
        versions = utils.get_backfill_versions_from_feature(
            feature,
            memberships,
        )
        for per_membership_cm_spec, v in zip(cm_specs_to_update, versions):
          per_membership_cm_spec.version = v
      for m, per_membership_cm_spec in zip(memberships, cm_specs_to_update):
        self._update_membership_feature(m, per_membership_cm_spec)
      return
    # --version must be set.
    _, installed_versions = utils.extract_membership_versions_from_feature(
        feature, memberships
    )
    for m, installed_v in zip(memberships, installed_versions):
      existing_cm_spec = self._cm_spec_for_membership(m)
      new_cm_spec = copy.deepcopy(existing_cm_spec)
      new_cm_spec.version = args.version
      if not self._is_config_sync_enabled(new_cm_spec):
        log.warning(
            "'configSync' field is not enabled for membership [%s]."
            " Configuration fields will not install and manage Config Sync on"
            " this cluster. See --help for details.", m
        )
      # Move under "if args.version" when other per-field flags are introduced.
      self._prompt_for_version_confirmation(
          m, existing_cm_spec, new_cm_spec, installed_v
      )
      self._update_membership_feature(m, new_cm_spec)

  def _update_membership_feature(self, membership: str, cm_spec):
    try:
      return self.UpdateV2(
          membership_path=membership,
          mask=['spec'],
          patch=self.messages_v2.MembershipFeature(
              spec=self.messages_v2.FeatureSpec(
                  configmanagement=cm_spec,
              ),
          ),
      )
    except apitools_exceptions.HttpNotFoundError as e:
      # TODO: b/431859521 - Remove this extra handling once Membership does
      # not exist error message is more readable. UpdateV2 creates
      # MembershipFeature if it does not exist, so NOT FOUND must mean
      # Membership does not exist.
      readable_e = api_lib_exceptions.HttpException(e, '{message}')
      # Do not use 'from readable_e' because readable_e won't display.
      raise exceptions.Error(
          f'membership [{membership}] not found: {readable_e}'
      )

  def _cm_spec_for_membership(self, membership: str):
    # CLH will error if cm_spec remains this default value because configSync
    # is not configured. Do not error here to not duplicate the CLH behavior,
    # which is also subject to change. The existing behavior is also preferred
    # if future per-field flags are introduced that populate the configSync
    # field.
    cm_spec = self.messages_v2.ConfigManagementSpec()
    try:
      mf = self.GetMembershipFeature(membership)
      if mf.spec and mf.spec.configmanagement:
        cm_spec = mf.spec.configmanagement
    except apitools_exceptions.HttpNotFoundError:
      pass
    return cm_spec

  def _prompt_for_version_confirmation(
      self, membership: str, existing_spec, new_spec, installed_version: str
  ):
    """Prompts for --version change confirmation.

    Should only be called when the --version flag is set and after all other
    flags update new_spec. Skips prompt if --version does not change the
    existing version or the configSync field is not enabled.

    Args:
      membership: Membership whose spec is being updated with --version.
      existing_spec: Existing spec for Membership. Version field may be None if
        unset.
      new_spec: Spec to update to with change from --version and all other
        flags.
      installed_version: Installed version in Membership state. May be empty
        string.
    """
    skip_prompt_msg = None
    # If --version="". defer to configSync enabled check, since empty string may
    # default to latest version if configSync enabled in same command call.
    if existing_spec.version == new_spec.version and new_spec.version:
      # Do not skip API call just because version flag specifies the existing
      # value. In the event of future per-field flags, we'd have to check the
      # entire spec for equality and handle edge cases. CLH may also introduce
      # side effects. We want minimize logic here.
      skip_prompt_msg = (
          f"--version='{new_spec.version}' is equivalent to the existing"
          f' configured version for membership [{membership}].'
      )
    elif not self._is_config_sync_enabled(new_spec):
      skip_prompt_msg = (
          "The 'version' field will not manage Config Sync because the"
          f" 'configSync' field is not enabled for membership [{membership}]."
          ' See --help for details.'
      )
    if skip_prompt_msg:
      if not properties.VALUES.core.disable_prompts.GetBool():
        log.status.Print(
            f'Note: Skipping --version confirmation prompt. {skip_prompt_msg}'
        )
      return
    if installed_version:
      # Note that installed version could be same as intended version.
      installed_version_msg = (
          f'The feature state detects version [{installed_version}] on this'
          ' cluster.'
      )
    else:
      # Cannot rely on lack of installed version to indicate lack of
      # Config Sync installation and skip prompt as a result because the spec
      # may have been empty or state updates may lag. This matters even more if
      # we introduce per-field flags --git or --oci that can enable the
      # configSync field.
      installed_version_msg = (
          'The feature state has yet to detect Config Sync on this cluster.'
      )
    version_msg = (
        'the latest version'
        if not new_spec.version
        else f'[{new_spec.version}]'
    )
    console_io.PromptContinue(
        message=(
            f'About to upgrade Config Sync on membership [{membership}] to'
            f' {version_msg}. {installed_version_msg} This command will not'
            ' block on upgrade completion.'
        ),
        # Declarative. Entire operation either succeeds or fails.
        cancel_on_no=True,
    )

  def _is_config_sync_enabled(self, cm_spec) -> bool:
    # Hopefully, we can avoid replicating the CLH logic in the future if, say,
    # the CLH backfills the enabled field based on git or oci presence.
    if not cm_spec.configSync:
      return False
    if cm_spec.configSync.enabled is not None:
      return cm_spec.configSync.enabled
    is_git_configured = (
        cm_spec.configSync.git is not None
        and cm_spec.configSync.git
        != self.messages_v2.ConfigManagementGitConfig()
    )
    is_oci_configured = (
        cm_spec.configSync.oci is not None
        and cm_spec.configSync.oci
        != self.messages_v2.ConfigManagementOciConfig()
    )
    return is_git_configured or is_oci_configured



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""List Google Compute Engine Organization Rollout Plans."""

from apitools.base.py import list_pager
from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.calliope import base


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
@base.UniverseCompatible
class List(base.ListCommand):
  """List Google Compute Engine Organization Rollout Plans."""

  detailed_help = {
      'brief': 'List Google Compute Engine Organization Rollout Plans.',
      'DESCRIPTION': (
          'List Google Compute Engine Organization Rollout Plans under'
          ' an organization.'
      ),
      'EXAMPLES': (
          r"""
    To list all organization rollout plans in organization '123456789', run:

      $ {command} --organization=123456789
    """
      ),
  }

  @staticmethod
  def Args(parser):
    parser.add_argument(
        '--organization',
        required=True,
        help='The Google Cloud organization ID.',
    )
    parser.display_info.AddFormat("""
      table(
        name,
        id,
        creationTimestamp:label=CREATION_TIME,
        locationScope:label=LOCATION_SCOPE,
        description
      )
    """)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client
    service = client.apitools_client.organizationRolloutPlans
    messages = holder.client.messages

    request = messages.ComputeOrganizationRolloutPlansListRequest(
        organization='organizations/' + args.organization,
    )

    return list_pager.YieldFromList(
        service,
        request,
        limit=args.limit,
        batch_size=args.page_size,
        method='List',
        field='items',
        current_token_attribute='pageToken',
        next_token_attribute='nextPageToken',
        batch_size_attribute='maxResults',
    )
